(* cw: a Morse code program
   Copyright (C) 2005  Eric C. Cooper <ecc@cmu.edu>
   Released under the GNU General Public License *)

let pi = 4.0 *. atan 1.0

let sample_rate = 8000 (* Hz; for /dev/dsp *)

let omega frequency = 2.0 *. pi *. float frequency /. float sample_rate

(* Convert a float in [-1,1] to an integer in (0 .. 255) *)

let signed_byte x = truncate (127.5 *. x +. 128.0)

let num_samples duration = sample_rate * duration / 1000

type t = Audio of string

let null = Audio ""

let tone ~hz ~ms =
  let n = num_samples ms in
  let om = omega hz in
  let data = String.create n in
  for i = 0 to n - 1 do
    data.[i] <- char_of_int (signed_byte (sin (om *. float i)))
  done;
  Audio data

let silence duration =
  Audio (String.make (num_samples duration) (char_of_int 0))

let audio_out = ref stdout

let init () =
  audio_out := open_out_bin "/dev/dsp"

(* Synchronize with sound card. *)

external sound_sync : Unix.file_descr -> unit = "sound_sync"

let sync () =
  flush !audio_out;
  sound_sync (Unix.descr_of_out_channel !audio_out)

let play (Audio str) = output_string !audio_out str
