(* cw: a Morse code program
   Copyright (C) 2005  Eric C. Cooper <ecc@cmu.edu>
   Released under the GNU General Public License *)

(* from the Emacs yow-file:

Zippy the pinhead data base.
Everything up to the first ascii \000 (`null') character is a comment.
The file consists of Zippy quotations (from various comic books and
 strips by Bill Griffith) followed by a null character.
 Newline characters following a quotation are ignored and are present
 only for readability.

*)

let file () =
  let chan =
    Unix.open_process_in
      "emacs --no-site-file --batch --load yow --eval '(princ yow-file)'"
  in
  let name = input_line chan in
  ignore (Unix.close_process_in chan);
  name

let read chan =
  let buf = Buffer.create 80 in
  let no_space () =
    let n = Buffer.length buf in
    n = 0 || Buffer.nth buf (n - 1) <> ' '
  in
  let rec loop () =
    let opt = try Some (input_char chan) with End_of_file -> None in
    match opt with
    | None when Buffer.length buf = 0 -> raise End_of_file
    | None | Some '\000' -> Buffer.contents buf
    | Some '\n' -> loop ()
    | Some ' ' ->
	if no_space () then Buffer.add_char buf ' ';
	loop ()
    | Some ch ->
	if Morse.valid_char ch then Buffer.add_char buf ch;
	loop ()
  in
  loop ()
